var sidarext_keyboardShortcutsConfigured = false;
var sidarext_localizedOptionsSetup     = false;
var sidarext_selectedTab         = -1;
var sidarext_tabSelectListener       = false;

sidarext_initialize();

// Displays the about dialog
function sidarext_about()
{
  window.openDialog("chrome://sidarext/content/about/about.xul", "sidarext-about-dialog", "centerscreen,chrome,modal");
}

// Changes the options
function sidarext_changeOptions(currentDocument)
{
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
  const toolbar      = document.getElementById("sidarext-toolbar");

  var element           = null;
  var hideMenuPreference    = false;
  var hideContextMenuPreference = false;

  // If the hide menu preference is set
  if(preferencesService.prefHasUserValue("sidarext.menu.hide"))
  {
    hideMenuPreference = preferencesService.getBoolPref("sidarext.menu.hide");
  }

  // If the hide context menu preference is set
  if(preferencesService.prefHasUserValue("sidarext.context.hide"))
  {
    hideContextMenuPreference = preferencesService.getBoolPref("sidarext.context.hide");
  }

  // If the toolbar exists
  if(toolbar)
  {
    const toolbarVisible = window.toolbar.visible;

    // If the toolbar is visible and the toolbar icons preference is set
    if(toolbarVisible && preferencesService.prefHasUserValue("sidarext.toolbar.icons"))
    {
      const toolbarButtons = toolbar.getElementsByTagName("toolbarbutton");

      var toolbarPreference = preferencesService.getComplexValue("sidarext.toolbar.icons", Components.interfaces.nsISupportsString).data.trim();

      toolbar.setAttribute("mode", toolbarPreference);

      // If the toolbar preference is set to full
      if(toolbarPreference == "full")
      {
        toolbarPreference = null;
      }
      else if(toolbarPreference == "icons")
      {
        toolbarPreference = "pictures";
      }

      toolbar.setAttribute("buttonstyle", toolbarPreference);

      // Loop through the toolbar buttons
      for(var i = 0; i < toolbarButtons.length; i++)
      {
        toolbarButtons[i].setAttribute("buttonstyle", toolbarPreference);
      }

      // If the toolbar preference is set to text
      if(toolbarPreference == "text")
      {
        element = document.getElementById("sidarext-toggle-statusbar");

        // If the element exists
        if(element)
        {
          const stringBundle = document.getElementById("sidarext-string-bundle");

          element.setAttribute("label", stringBundle.getString("sidarext_toggleLabel"));
        }
      }
      else
      {
        element = document.getElementById("sidarext-javascript-statusbar");

        // If the element exists
        if(element)
        {
          element.removeAttribute("label");
        }

        element = document.getElementById("sidarext-render-mode-statusbar");

        // If the element exists
        if(element)
        {
          element.removeAttribute("label");
        }

        element = document.getElementById("sidarext-toggle-statusbar");

        // If the element exists
        if(element)
        {
          element.removeAttribute("label");
        }
      }
    }
  }

  element = document.getElementById("sidarext-menu");

  // If the element exists
  if(element)
  {
    element.setAttribute("hidden", hideMenuPreference);
  }

  element = document.getElementById("sidarext-context");

  // If the element exists
  if(element)
  {
    element.setAttribute("hidden", hideContextMenuPreference);
  }

  element = document.getElementById("sidarext-context-separator");

  // If the element exists
  if(element)
  {
    element.setAttribute("hidden", hideContextMenuPreference);
  }

  // If the keyboard shortcuts have not been configured
  if(!sidarext_keyboardShortcutsConfigured)
  {
    sidarext_keyboardShortcutsConfigured = sidarext_configureKeyboardShortcuts(false);
  }
}

// Configure the keyboard shortcuts
function sidarext_configureKeyboardShortcuts(reset)
{
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
  const string       = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
  const windowEnumeration  = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator).getEnumerator("navigator:browser");

  var mainDocument = null;
  var success    = false;

  // Loop through the open windows
  while(windowEnumeration.hasMoreElements())
  {
    mainDocument = windowEnumeration.getNext().document;

    // If a main document was found
    if(mainDocument)
    {
      var id         = null;
      var key        = null;
      var keyPreference    = null;
      var keyPreferenceValue = null;
      var keys         = null;
      var keySet       = mainDocument.getElementById("mainKeyset");
      var toolCount      = 0;
      var url        = null;

      // If the key set was not found
      if(!keySet)
      {
        keySet = mainDocument.getElementById("navKeys");
      }

      // If the key set was found
      if(keySet)
      {
        keys = keySet.childNodes;

        // Loop through the keys
        for(var i = 0; i < keys.length; i++)
        {
          key = keys.item(i);

          // If the key has an id starting with sidarext
          if(key.hasAttribute("id") && key.getAttribute("id").indexOf("sidarext") == 0)
          {
            id = key.getAttribute("id").split("-").join(".");

            // If the key preference is set
            if(preferencesService.prefHasUserValue(id))
            {
              key.setAttribute("key", preferencesService.getComplexValue(id, Components.interfaces.nsISupportsString).data.trim());
            }
            else if(!reset)
            {
              string.data = key.getAttribute("key");

              preferencesService.setComplexValue(id, Components.interfaces.nsISupportsString, string);
            }

            success = true;
          }
        }

        // If the tool count preference is set
        if(preferencesService.prefHasUserValue("sidarext.tool.count"))
        {
          toolCount = preferencesService.getIntPref("sidarext.tool.count");
        }

        // Loop through the possible tools
        for(i = 1; i <= toolCount; i++)
        {
          keyPreference = "sidarext.tool." + i + ".key";
          url       = "sidarext.tool." + i + ".url";

          // If the URL and key preferences are set
          if(preferencesService.prefHasUserValue(url) && preferencesService.prefHasUserValue(keyPreference))
          {
            keyPreferenceValue = preferencesService.getComplexValue(keyPreference, Components.interfaces.nsISupportsString).data.trim();

            // If the key preference value is set
            if(keyPreferenceValue)
            {
              key = mainDocument.createElement("key");

              key.setAttribute("id", keyPreference);
              key.setAttribute("key", keyPreferenceValue);
              key.setAttribute("modifiers", "accel,shift");
              key.setAttribute("oncommand", "sidarext_loadURL('" + preferencesService.getComplexValue(url, Components.interfaces.nsISupportsString).data.trim() + "' + escape(getBrowser().currentURI.spec))");
              keySet.appendChild(key);
            }
          }
        }
      }
    }
  }

  return success;
}


// Disables an element
function sidarext_disableElement(element, disable)
{
  // If the element exists
  if(element)
  {
    element.setAttribute("disabled", disable);
  }
}

// Disables the given preference
function sidarext_disablePreference(element, preference)
{
  const checked      = element.getAttribute("checked");
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  preferencesService.setBoolPref(preference, !checked);
  sidarext_configureEquivalentElements(element, "checked", checked);
}

// Displays the elements ancestors in the status bar
function sidarext_displayElementAncestors(event)
{
  const eventTarget = event.target;

  // If there is a target
  if(eventTarget)
  {
    getBrowser().contentWindow.status = sidarext_getElementAncestors(eventTarget, "");

    event.preventDefault();
  }
}

// Enables the given preference
function sidarext_enablePreference(element, preference)
{
  const checked      = element.getAttribute("checked");
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  // If the element is checked - explicit if required
  if(checked)
  {
    preferencesService.setBoolPref(preference, true);
  }
  else
  {
    preferencesService.setBoolPref(preference, false);
  }

  sidarext_configureEquivalentElements(element, "checked", checked);
}

// Generates a page in a new tab or window
function sidarext_generatePage(url)
{
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  var generatedPage = null;

  // If the open tabs preference is set to true
  if(preferencesService.prefHasUserValue("sidarext.open.tabs") && preferencesService.getBoolPref("sidarext.open.tabs"))
  {
    const newTab = getBrowser().addTab(url);

    getBrowser().selectedTab = newTab;
    generatedPage = window;
  }
  else
  {
    generatedPage = window.open(url);
  }

  return generatedPage;
}

// Gets all the documents from the current page
function sidarext_getDocuments(frame, documentList)
{
  const framesList = frame.frames;

  documentList.push(frame.document);

  // Loop through the frames
  for(var i = 0; i < framesList.length; i++)
  {
    sidarext_getDocuments(framesList[i], documentList);
  }

  return documentList;
}

// Get an elements ancestors
function sidarext_getElementAncestors(element, ancestorList)
{
  const classAttribute = element.getAttribute("class");
  const idAttribute  = element.getAttribute("id");
  const parentElement  = element.parentNode;

  var elementText = element.tagName.toLowerCase();

  // If there is a class attribute
  if(classAttribute)
  {
    elementText += "." + classAttribute;
  }

  // If there is an id attribute
  if(idAttribute)
  {
    elementText += "#" + idAttribute;
  }

  // If there is an ancestor list
  if(ancestorList)
  {
    elementText += " > " + ancestorList
  }

  // If there is a parent element with a tag name
  if(parentElement && parentElement.tagName)
  {
    return sidarext_getElementAncestors(parentElement, elementText);
  }
  else
  {
    return elementText;
  }
}

// Initializes the extension
function sidarext_initialize()
{
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  sidarext_setupDefaultOptions();

  // Try to remove the load event listener
  try
  {
    window.removeEventListener("load", sidarext_pageLoad, true);
  }
  catch(exception)
  {
    // Do nothing
  }

  // Try to remove the unload event listener
  try
  {
    window.removeEventListener("unload", sidarext_pageUnload, true);
  }
  catch(exception)
  {
    // Do nothing
  }

  window.addEventListener("load", sidarext_pageLoad, true);
  window.addEventListener("unload", sidarext_pageUnload, true);
}

// Loads the given URL
function sidarext_loadURL(url)
{
  const oldTab       = getBrowser().selectedTab;
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  sidarext_generatePage(url);

  // If the open tabs in background preference is set to true
  if(preferencesService.prefHasUserValue("sidarext.open.tabs.background") && preferencesService.getBoolPref("sidarext.open.tabs.background"))
  {
    getBrowser().selectedTab = oldTab;
  }
}

// Displays the options dialog
function sidarext_options()
{
  window.openDialog("chrome://sidarext/content/options/options.xul", "sidarext-options-dialog", "centerscreen,chrome,modal");

  sidarext_changeOptions();
}

// Handles the page being loaded
function sidarext_pageLoad(event)
{
  const eventTarget    = event.target;
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  var disable = false;

  // If the disabled preference is set
  if(preferencesService.prefHasUserValue("sidarext.disabled"))
  {
    disable = preferencesService.getBoolPref("sidarext.disabled");
  }

  // If the page is the target
  if(eventTarget && eventTarget.hasAttribute && eventTarget.hasAttribute("id") && eventTarget.getAttribute("id").toLowerCase() == "content")
  {
    const stringBundle = document.getElementById("sidarext-string-bundle");

    sidarext_changeOptions();

    // If the extension should be disabled, but is not
    if(disable && document.getElementById("sidarext-toggle-statusbar").getAttribute("tooltiptext") == stringBundle.getString("sidarext_disable"))
    {
      sidarext_disable(true);
    }
    else if(!disable)
    {
      const content             = window.document.getElementById("content");
      const contentDocument     = eventTarget.contentDocument;
      const loadedDocument      = event.originalTarget;

      var element              = null;

      // If the localized options have not been setup
      if(!sidarext_localizedOptionsSetup)
      {
        sidarext_setupLocalizedOptions();
      }

      // If the content exists
      if(!sidarext_tabSelectListener && content)
      {
        const tabBox = content.mTabBox;

        // Try to remove the event listener
        try
        {
          tabBox.removeEventListener("select", sidarext_tabSelect, true);
        }
        catch(exception)
        {
          // Do nothing
        }

        tabBox.addEventListener("select", sidarext_tabSelect, true);
        sidarext_tabSelectListener = true;
      }
    }
  }
}

// Handles the page being unloaded
function sidarext_pageUnload(event)
{
  const eventTarget  = event.target;
  const originalTarget = event.originalTarget;

}


// Removes all the generated menu items from the menu
function sidarext_removeGeneratedMenuItems(menu)
{
  var menuItem  = null;
  var menuItems = menu.getElementsByAttribute("class", "sidarext-generated-menu");

  // Loop through the menu items
  for(var i = 0; i < menuItems.length; i++)
  {
    menu.removeChild(menuItems[i]);
  }

  menuItems = menu.getElementsByAttribute("class", "sidarext-generated-menu");

  // Loop through the menu items again because of the change to the behavior of getElementsByAttribute
  while(menuItems.length > 0)
  {
    menu.removeChild(menuItems[0]);
  }
}


// Sets up the default options
function sidarext_setupDefaultOptions()
{
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
  const string       = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);

  // If the open tabs preference is not set
  if(!preferencesService.prefHasUserValue("sidarext.open.tabs"))
  {
    // If the open windows preference is set
    if(preferencesService.prefHasUserValue("sidarext.open.windows"))
    {
      preferencesService.setBoolPref("sidarext.open.tabs", !preferencesService.getBoolPref("sidarext.open.windows"));
      preferencesService.clearUserPref("sidarext.open.windows");
    }
    else
    {
      preferencesService.setBoolPref("sidarext.open.tabs", true);
    }
  }

}

// Sets up the localized options
function sidarext_setupLocalizedOptions()
{
  const stringBundle = document.getElementById("sidarext-string-bundle");

  // If the string bundle is set
  if(stringBundle)
  {
    const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
    const string       = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);


    // Set default doc count
    if(!preferencesService.prefHasUserValue("sidarext.doc.count"))
    {
      preferencesService.setIntPref("sidarext.doc.count", 0);
    }

    // Set default blog count
    if(!preferencesService.prefHasUserValue("sidarext.blog.count"))
    {
      preferencesService.setIntPref("sidarext.blog.count", 0);
    }

    // Set default list count
    if(!preferencesService.prefHasUserValue("sidarext.list.count"))
    {
      preferencesService.setIntPref("sidarext.list.count", 1);
    }
    // Set Ovillo list
    if(!preferencesService.prefHasUserValue("sidarext.list.1.description"))
    {
      string.data = stringBundle.getString("sidarext_listOvillo");

      preferencesService.setComplexValue("sidarext.list.1.description", Components.interfaces.nsISupportsString, string);
    }
    if(!preferencesService.prefHasUserValue("sidarext.list.1.url"))
    {
      string.data = "http://www.mail-archive.com/ovillo@ovillo.org/";

      preferencesService.setComplexValue("sidarext.list.1.url", Components.interfaces.nsISupportsString, string);
    }


    // Set default resource count
    if(!preferencesService.prefHasUserValue("sidarext.resource.count"))
    {
      preferencesService.setIntPref("sidarext.resource.count", 0);
    }

    // Set default tool count
    if(!preferencesService.prefHasUserValue("sidarext.tool.count"))
    {
      preferencesService.setIntPref("sidarext.tool.count", 0);
    }


    sidarext_localizedOptionsSetup = true;
  }
}


// Handles a tab being selected
function sidarext_tabSelect(event)
{
  const selectedTab   = getBrowser().mTabBox.selectedIndex;
  const currentDocument = getBrowser().browsers[selectedTab].contentWindow.document;

  // If the selected tab is different
  if(selectedTab != sidarext_selectedTab)
  {
    const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

    sidarext_selectedTab = selectedTab;

    sidarext_updateRenderMode(currentDocument);
  }
}


